/*
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 3 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#include "callbacks.h"
#include "shared.h"

/* For clearing text buffers */
static GtkTextBuffer *text_buffer;

/* Various dialogue widgets */
static GtkWidget
  *quit_dialog  = NULL,
  *save_dialog  = NULL,
  *error_dialog = NULL,
  *bad_entry_dialog = NULL;



/*  Error_Dialog()
 *
 *  Opens an error dialog box
 */

  void
Error_Dialog( char *mesg, gboolean hide )
{
  GtkBuilder *builder = NULL;
  if( error_dialog != NULL ) return;
  error_dialog = create_error_dialog( &builder );
  GtkWidget *lbl = Builder_Get_Object( builder, "error_message" );
  gtk_label_set_text( GTK_LABEL(lbl), mesg );
  if( hide )
	gtk_widget_hide( Builder_Get_Object(builder, "error_ok_button") );
  gtk_widget_show( error_dialog );
}


  void
Quit_Dialog( char *mesg )
{
  GtkBuilder *builder = NULL;
  quit_dialog = create_quit_dialog( &builder );
  GtkWidget *lbl = Builder_Get_Object( builder, "quit_message" );
  gtk_label_set_text( GTK_LABEL(lbl), mesg );
  gtk_widget_show( quit_dialog );
}

  void
Save_Dialog( char *mesg )
{
  GtkBuilder *builder = NULL;
  save_dialog = create_save_dialog( &builder );
  GtkWidget *lbl = Builder_Get_Object( builder, "save_message" );
  gtk_label_set_text( GTK_LABEL(lbl), mesg );
  gtk_widget_show( save_dialog );
}


  void
Bad_Entry_Dialog( char *mesg )
{
  GtkBuilder *builder = NULL;
  bad_entry_dialog = create_bad_entry_dialog( &builder );
  GtkWidget *lbl = Builder_Get_Object( builder, "bad_entry" );
  gtk_label_set_text( GTK_LABEL(lbl), mesg );
  gtk_widget_show( bad_entry_dialog );
}


void
on_main_window_destroy(
	GObject  *object,
	gpointer  user_data)
{
  Cleanup();
  gtk_main_quit();
}


gboolean
on_main_window_delete_event(
	GtkWidget *widget,
	GdkEvent  *event,
	gpointer   user_data)
{
  if( isFlagSet(TRANSMIT_MODE) ||
	  isFlagSet(RECEIVE_MODE) )
  {
	Quit_Dialog(
		_("xpsk31 is running\n"\
		  "Still want to quit?") );
	return( TRUE );
  }
  else return( FALSE );
}


gboolean
on_tx_textview_button_press_event(
	GtkWidget      *widget,
	GdkEventButton *event,
	gpointer        user_data)
{
  switch( event->button )
  {
	case 1: return( FALSE );

	case 2: /* Toggle Tx/Rx/Macro modes */
	  if( isFlagSet(TRANSMIT_MACRO) )
		ClearFlag( TRANSMIT_MACRO  );
	  else if( isFlagSet(TRANSMIT_KEYBD) )
	  {
		ClearFlag( TRANSMIT_KEYBD  );
		SetFlag( RECEIVE_MODE );
		g_idle_add( Receive_Mode, NULL );
	  }
	  else
	  {
		SetFlag( TRANSMIT_KEYBD );
		ClearFlag( RECEIVE_MODE );
		Set_TxRx_Labels();
		g_idle_add( Transmit_Keybd, "dummy" );
	  }
	  break;

	case 3: /* Popup menu */
	  if( isFlagSet(PLAYBACK_SETUP) &&
		  isFlagSet(TRANSMIT_KEYBD) )
		gtk_widget_set_sensitive( Builder_Get_Object(
			  popup_menu_builder, "identify_in_cw"), TRUE );
	  else
		gtk_widget_set_sensitive( Builder_Get_Object(
			  popup_menu_builder, "identify_in_cw"), FALSE );

	  gtk_menu_popup_at_pointer( GTK_MENU(popup_menu), NULL );

	  /* Needed for 'Clear Window' */
	  text_buffer = tx_text_buffer;

  } /* switch( event->button ) */

  return( TRUE );
}


gboolean
on_rx_textview_button_press_event(
	GtkWidget      *widget,
	GdkEventButton *event,
	gpointer        user_data)
{
  switch( event->button )
  {
	case 1: return( FALSE );

	case 2: /* Toggle Tx/Rx/Macro modes */
	  if( isFlagSet(RECEIVE_MODE) )
	  {
		ClearFlag( RECEIVE_MODE );
		Set_TxRx_Labels();
	  }
	  else
	  {
		if( isFlagClear(TRANSMIT_MACRO) )
		{
		  SetFlag( RECEIVE_MODE );
		  ClearFlag( TRANSMIT_KEYBD );
		  g_idle_add( Receive_Mode, NULL );
		}
		else
		{
		  ClearFlag( TRANSMIT_MACRO );
		  SetFlag( TRANSMIT_KEYBD );
		  Set_TxRx_Labels();
		}

	  } /* if( isFlagSet(RECEIVE_MODE) ) */
	  break;

	case 3: /* Popup menu */
	  if( isFlagSet(PLAYBACK_SETUP) &&
		  isFlagSet(TRANSMIT_KEYBD) )
		gtk_widget_set_sensitive( Builder_Get_Object(
			  popup_menu_builder, "identify_in_cw"), TRUE );
	  else
		gtk_widget_set_sensitive( Builder_Get_Object(
			  popup_menu_builder, "identify_in_cw"), FALSE );

	  gtk_menu_popup_at_pointer( GTK_MENU(popup_menu), NULL );

	  /* Needed for 'Clear Window' */
	  text_buffer = rx_text_buffer;

  } /* switch( event->button ) */

  return( TRUE );
}


void
on_squelch_value_changed(
	GtkRange *range,
	gpointer  user_data)
{
  GtkWidget *hscale;
  GtkAdjustment *adjustment;

  /* Read and enter squelch level */
  hscale = Builder_Get_Object( main_window_builder, "squelch_value" );
  adjustment = gtk_range_get_adjustment( GTK_RANGE(hscale) );
  rc_data.sqlch_thr =
	(int)(gtk_adjustment_get_value(adjustment)) * SQUELCH_MULTIPLIER;
}


void
on_op_name_changed(
	GtkEditable *editable,
	gpointer     user_data)
{
  char buff[13];
  int len;

  /* Read entry, ignore blank field */
  Strlcpy( buff,
	  gtk_entry_get_text(GTK_ENTRY(editable)),
	  sizeof(buff) );
  len = (int)strlen(buff);
  if( len == 0 ) return;

  /* Capitalize letters if enabled */
  if( isFlagSet(CAPITAL_LETTERS) )
  {
	int idx;
	for( idx = 0; idx < len; idx++ )
	  if( (buff[idx] > 0x60) &&
		  (buff[idx] < 0x7b) )
		buff[idx] -= 0x20;
  }

  /* Enter data to field QSO record structure */
  Strlcpy( qso_record.dx_name, buff, sizeof(qso_record.dx_name) );
  gtk_entry_set_text( GTK_ENTRY(editable), qso_record.dx_name );
}


void
on_qth_name_changed(
	GtkEditable *editable,
	gpointer     user_data)
{
  char buff[13];
  int len;

  /* Read entry, ignore blank field */
  Strlcpy( buff,
	  gtk_entry_get_text(GTK_ENTRY(editable)),
	  sizeof(buff) );
  len = (int)strlen(buff);
  if( len == 0 )
	return;

  /* Capitalize letters if enabled */
  if( isFlagSet(CAPITAL_LETTERS) )
  {
	int idx;
	for( idx = 0; idx < len; idx++ )
	  if( (buff[idx] > 0x60) &&
		  (buff[idx] < 0x7b) )
		buff[idx] -= 0x20;
  }

  /* Enter data to field and QSO record structure */
  Strlcpy( qso_record.dx_qth, buff, sizeof(qso_record.dx_qth) );
  gtk_entry_set_text( GTK_ENTRY(editable), qso_record.dx_qth );
}


void
on_locator_changed(
	GtkEditable *editable,
	gpointer     user_data)
{
  int len, idx;
  char buff[7];

  /* Get entry field, ignore blank field */
  Strlcpy( buff,
	  gtk_entry_get_text(GTK_ENTRY(editable)),
	  sizeof(buff) );
  len = (int)strlen(buff);
  if( len == 0 ) return;

  for( idx = 0; idx < len; idx++ )
  {
	/* Capitalize letters */
	if( (buff[idx] > 0x60) &&
		(buff[idx] < 0x7b) )
	  buff[idx] -= 0x20;

	/* Validate grid locator */
	switch( idx )
	{
	  case 0: case 1: /* First letters */
		if( (buff[idx] < 'A') || (buff[idx] > 'S') )
		{
		  Bad_Entry_Dialog(
			  _("Invalid character\n"\
				"Letters A-S only") );
		  return;
		}
		break;

	  case 2: case 3: /* Middle numbers */
		if( (buff[idx] < '0') || (buff[idx] > '9') )
		{
		  Bad_Entry_Dialog(
			  _("Invalid character\n"\
				"Numbers 0-9 only") );
		  return;
		}
		break;

	  case 4: case 5: /* Last letters */
		if( (buff[idx] < 'A') || (buff[idx] > 'X') )
		{
		  Bad_Entry_Dialog(
			  _("Invalid character\n"\
				"Letters A-X only") );
		  return;
		}

	} /* switch( idx ) */

  } /* for( idx = 0; idx < len; idx++ ) */

  /* Enter data to field and QSO record structure */
  Strlcpy( qso_record.dx_loc, buff, sizeof(qso_record.dx_loc) );
  gtk_entry_set_text( GTK_ENTRY(editable), qso_record.dx_loc );
}


void
on_band_changed(
	GtkEditable *editable,
	gpointer     user_data)
{
  int len, idx;
  char buff[14];

  /* Get entry field, ignore blank field */
  Strlcpy( buff,
	  gtk_entry_get_text(GTK_ENTRY(editable)),
	  sizeof(buff) );
  len = (int)strlen(buff);
  if( len == 0 ) return;

  /* Reject non-numbers except '.' and space */
  for( idx = 0; idx < len; idx++ )
	if( ((buff[idx]  < '0') || (buff[idx]  > '9')) &&
		((buff[idx] != '.') && (buff[idx] != ' ')) )
	{
	  Bad_Entry_Dialog(
		  _("Invalid character entered\n"\
			"Numbers and '.' only allowed") );
	  return;
	}

  /* Enter data to QSO record structure */
  Strlcpy( qso_record.freq, buff, sizeof(qso_record.freq) );
}


void
on_callsign_changed(
	GtkEditable *editable,
	gpointer     user_data)
{
  int len, idx, idy;
  char buff[15];

  /* Get entry field, ignore blank field */
  Strlcpy( buff,
	  gtk_entry_get_text(GTK_ENTRY(editable)),
	  sizeof(buff) );
  len = (int)strlen( buff );
  if( len == 0 ) return;

  for( idx = 0; idx < len; idx++ )
  {
	/* Capitalize letters */
	if( (buff[idx] > 0x60) &&
		(buff[idx] < 0x7b) )
	  buff[idx] -= 0x20;

	/* Allow only alpha-numerics and '/' */
	if( ((buff[idx]  < 'A') || (buff[idx]  > 'Z')) &&
		((buff[idx]  < '0') || (buff[idx]  > '9')) &&
		((buff[idx] != '/') && (buff[idx] != ' ')) )
	{
	  Bad_Entry_Dialog(
		  _("Invalid character entered\n"\
			"Alpha-numerics and '/' only") );
	  return;
	}

  } /* for( idx = 0; idx < len; idx++ ) */

  /* Enter data to field and QSO record structure */
  idy = 0;
  for( idx = 0; idx < len; idx++ )
	if( buff[idx] != ' ' ) /* ignore space */
	  qso_record.dx_call[idy++] = buff[idx];
  qso_record.dx_call[idy] = '\0';
  gtk_entry_set_text( GTK_ENTRY(editable), qso_record.dx_call );

  SetFlag( SAVE_RECORD );
}


void
on_new_record_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  /* Variables for time and date */
  time_t tp;    /* Time type     */
  struct tm dt; /* Date and time */

  /* For reading Tcvr status */
  char freq[8], rst[4];

  /* Save record if needed and completed */
  if( isFlagSet(SAVE_RECORD) )
  {
	if( !Read_QSO_Record() )
	{
	  Bad_Entry_Dialog(
		  _("QSO Record is not complete\n"\
			"Cannot save and open new Record") );
	  return;
	}
	else Save_QSO_Record();
  }

  /* Initialize for new entry */
  Clear_Record_Fields( FALSE );

  /* Read Receiver status if CAT enabled */
  if( isFlagSet(CAT_SETUP) )
  {
	GtkWidget *entry;

	if( !Read_Rx_Status(freq, rst) ) return;
	entry = Builder_Get_Object( main_window_builder, "rst_out" );
	gtk_entry_set_text( GTK_ENTRY(entry), rst );
	entry = Builder_Get_Object( main_window_builder, "band" );
	gtk_entry_set_text( GTK_ENTRY(entry), freq );
  }

  /* Enter time and date to QSO record */
  time( &tp );
  dt = *gmtime( &tp );
  strftime( qso_record.date,     12, "%d/%b/%Y", &dt );
  strftime( qso_record.time,      6, "%H:%M",    &dt );
  strftime( qso_record.date_adif, 9, "%Y%m%d",   &dt );
  strftime( qso_record.time_adif, 5, "%H%M",     &dt );

  /* Enter mode of operation */
  if( isFlagSet(MODE_BPSK) )
	Strlcpy( qso_record.mode, "BPSK", sizeof(qso_record.mode) );
  else
	Strlcpy( qso_record.mode, "QPSK", sizeof(qso_record.mode) );
}


void
on_save_record_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  /* Save record if needed (and valid) */
  if( Read_QSO_Record() )
	Save_Dialog( _("Save QSO Record to file?") );
  else Save_Dialog(
	  _("QSO Record is incomplete\n"\
		"Do you still want to save?") );
}


  void
on_clear_fields_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Clear_Record_Fields(TRUE);
}


void
on_error_quit_button_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Cleanup();
  gtk_main_quit();
}


void
on_cancel_save_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  gtk_widget_destroy( save_dialog );
}


void
on_save_ok_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  gtk_widget_destroy( save_dialog );
  Save_QSO_Record();
}

void
on_bpsk_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	SetFlag( MODE_BPSK );
	ClearFlag( MODE_QPSK_USB | MODE_QPSK_LSB );
	Set_Tcvr_Mode();
	Set_TxRx_Labels();
  }
}


void
on_qpsk_lsb_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	SetFlag( MODE_QPSK_LSB );
	ClearFlag( MODE_QPSK_USB | MODE_BPSK );
	Set_Tcvr_Mode();
	Set_TxRx_Labels();
	Init_Viterbi();
  }
}


void
on_qpsk_usb_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	SetFlag( MODE_QPSK_USB );
	ClearFlag( MODE_QPSK_LSB | MODE_BPSK );
	Set_Tcvr_Mode();
	Set_TxRx_Labels();
	Init_Viterbi();
  }
}


void
on_capitalize_letters_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
	SetFlag( CAPITAL_LETTERS );
  else
	ClearFlag( CAPITAL_LETTERS );
}


void
on_quit_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( isFlagSet(TRANSMIT_MODE) ||
	  isFlagSet(RECEIVE_MODE) )
	Quit_Dialog(
		_("xpsk31 is running\n"\
		  "Still want to quit?") );
  else gtk_main_quit();
}


  void
on_record_qsos_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
	SetFlag( RECORD_QSO );
  else ClearFlag( RECORD_QSO );
}

  void
on_identify_in_cw_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  SetFlag( TRANSMIT_MORSE_ID );
}


void
on_bad_entry_ok_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  gtk_widget_destroy( bad_entry_dialog );
}


void
on_cancel_quit_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  gtk_widget_destroy( quit_dialog );
}


void
on_quit_ok_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Cleanup();
  gtk_main_quit();
}


void
on_clear_window_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  gtk_text_buffer_set_text( text_buffer, "", -1 );
}


gboolean
on_tx_textview_key_press_event(
	GtkWidget   *widget,
	GdkEventKey *event,
	gpointer     user_data)
{
  /* Text buffer marker */
  static GtkTextIter iter;

  if( (event->keyval == GDK_KEY_Shift_L) ||
	  (event->keyval == GDK_KEY_Shift_R) )
	return( FALSE );

  /* Save key press character */
  key_buff[key_cnt] = event->keyval;

  /* Print only if not in Tx mode */
  if( isFlagClear(TRANSMIT_MODE) )
  {
	/* Capitalize letters if enabled */
	if( isFlagSet(CAPITAL_LETTERS) &&
		(key_buff[key_cnt] > 0x60) &&
		(key_buff[key_cnt] < 0x7b) )
	  key_buff[key_cnt] -= 0x20;

	/* Set up TRANSMIT text view */
	gtk_text_buffer_get_iter_at_offset(
		tx_text_buffer, &iter,
		gtk_text_buffer_get_char_count
		(tx_text_buffer) );

	/* Reset word wrap on return key */
	if( key_buff[key_cnt] == GDK_KEY_Return )
	  char_cnt = 0;

	/* Do word wrapping */
	char_cnt++;
	if( (char_cnt >= rc_data.wwrap) &&
		(key_buff[key_cnt] == ' ') )
	{
	  /* Reset word wrap */
	  char_cnt = 0;

	  Print_Character(
		  GDK_KEY_Return, tx_text_buffer,
		  tx_scrolledwindow, &iter );

	  /* Increment/Reset keystroke count */
	  if( ++key_cnt >= KEY_BUFF_SIZE )
		key_cnt = 0;

	  return( TRUE );
	} /* if( (char_cnt >= rc_data.wwrap) && */

	Print_Character(
		(int)key_buff[key_cnt],
		tx_text_buffer,
		tx_scrolledwindow,
		&iter );

  } /* if( isFlagClear(TRANSMIT_MODE) ) */

  /* Increment/Reset keystroke count */
  if( ++key_cnt >= KEY_BUFF_SIZE )
	key_cnt = 0;

  return( TRUE );
}

  gboolean
on_scope_draw(
	GtkWidget *widget,
	cairo_t   *cr,
	gpointer   user_data)
{
  if( isFlagClear(ENABLE_MAGNIPHASE) )
  {
	if( wfall_pixbuf != NULL )
	{
	  gdk_cairo_set_source_pixbuf( cr, wfall_pixbuf, 0.0, 0.0 );
	  cairo_paint( cr );
	}
  }
  else Update_Magniphase( cr );

  return( TRUE );
}


  gboolean
on_scope_button_press_event(
	GtkWidget      *widget,
	GdkEventButton *event,
	gpointer        user_data)
{
  if( event->button == 1 )
  {
	if( isFlagSet(CAT_SETUP)    &&
		isFlagSet(RECEIVE_MODE) &&
		isFlagClear(ENABLE_MAGNIPHASE) )
	  Tune_Tcvr( event->x );
  }
  else
  {
	if( isFlagSet( ENABLE_MAGNIPHASE ) )
	{
	  gtk_label_set_text( GTK_LABEL(scope_label), _("Waterfall") );
	  ClearFlag( ENABLE_MAGNIPHASE );
	}
	else
	{
	  gtk_label_set_text( GTK_LABEL(scope_label), _("Magniphase") );
	  SetFlag( ENABLE_MAGNIPHASE );
	  freq_error = 0.0;
	}
  }

  return( TRUE );
}


void
on_clear_windows_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  gtk_text_buffer_set_text(tx_text_buffer, "", -1);
  gtk_text_buffer_set_text(rx_text_buffer, "", -1);
}


void
on_freq_adjust_value_changed(
	GtkRange *range,
	gpointer  user_data)
{
  GtkWidget     *hscale;
  GtkAdjustment *adjustment;

  /* Read and enter freq correction */
  hscale = Builder_Get_Object( main_window_builder, "freq_adjust" );
  adjustment = gtk_range_get_adjustment( GTK_RANGE(hscale) );
  freq_error = gtk_adjustment_get_value( adjustment );
}


  void
on_k2_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	rc_data.tcvr_type = K2;
	SetFlag( ENABLE_CAT );
	ClearFlag( ENABLE_RTS_DTR );
	Close_Tcvr_Serial();
  }
}


  void
on_k3_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	rc_data.tcvr_type = K3;
	SetFlag( ENABLE_CAT );
	ClearFlag( ENABLE_RTS_DTR );
	Close_Tcvr_Serial();
  }
}


void
on_ft847_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	rc_data.tcvr_type = FT847;
	SetFlag( ENABLE_CAT );
	ClearFlag( ENABLE_RTS_DTR );
	Close_Tcvr_Serial();
  }
}


void
on_ft857_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	rc_data.tcvr_type = FT857;
	SetFlag( ENABLE_CAT );
	ClearFlag( ENABLE_RTS_DTR );
	Close_Tcvr_Serial();
  }
}


  void
on_rts_ctl_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	rc_data.tcvr_type = NONE;
	SetFlag( ENABLE_RTS_DTR );
	ClearFlag( ENABLE_CAT );
	Close_Tcvr_Serial();
  }
}


void
on_none_activate(
	GtkMenuItem *menuitem,
	gpointer     user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	rc_data.tcvr_type = NONE;
	ClearFlag( ENABLE_CAT );
	ClearFlag( ENABLE_RTS_DTR );
	Close_Tcvr_Serial();
  }
}


void
on_xmit_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  /* Toggle Tx/Rx/Macro modes */
  if( isFlagSet(TRANSMIT_MACRO) )
	ClearFlag( TRANSMIT_MACRO  );
  else if( isFlagSet(TRANSMIT_KEYBD) )
  {
	ClearFlag( TRANSMIT_KEYBD  );
	SetFlag( RECEIVE_MODE );
	g_idle_add( Receive_Mode, NULL );
  }
  else
  {
	SetFlag( TRANSMIT_KEYBD );
	ClearFlag( RECEIVE_MODE );
	Set_TxRx_Labels();
	g_idle_add( Transmit_Keybd, "dummy" );
  }

  return;
}


void
on_rcve_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  /* Toggle Tx/Rx/Macro modes */
  if( isFlagSet(RECEIVE_MODE) )
  {
	ClearFlag( RECEIVE_MODE );
	Set_TxRx_Labels();
  }
  else
  {
	if( isFlagClear(TRANSMIT_MACRO) )
	{
	  SetFlag( RECEIVE_MODE );
	  ClearFlag( TRANSMIT_KEYBD );
	  g_idle_add( Receive_Mode, NULL );
	}
	else
	{
	  ClearFlag( TRANSMIT_MACRO );
	  SetFlag( TRANSMIT_KEYBD );
	  Set_TxRx_Labels();
	}
  } /* if( isFlagSet(RECEIVE_MODE) ) */

  return;
}


void
on_rst_out_changed(
	GtkEditable *editable,
	gpointer     user_data)
{
  Process_RST( editable );
  Strlcpy( qso_record.dx_rst,
	  gtk_entry_get_text(GTK_ENTRY(editable)),
	  sizeof(qso_record.dx_rst) );
}


void
on_rst_in_changed(
	GtkEditable *editable,
	gpointer     user_data)
{
  Process_RST( editable );
  Strlcpy( qso_record.my_rst,
	  gtk_entry_get_text(GTK_ENTRY(editable)),
	  sizeof(qso_record.my_rst) );
}


void
on_macro1_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Select_Macro( 1 );
}


void
on_macro2_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Select_Macro( 2 );
}


void
on_macro3_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Select_Macro( 3 );
}


void
on_macro4_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Select_Macro( 4 );
}


void
on_macro5_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Select_Macro( 5 );
}


void
on_macro6_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Select_Macro( 6 );
}


void
on_macro7_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Select_Macro( 7 );
}


void
on_macro8_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Select_Macro( 8 );
}


void
on_macro9_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Select_Macro( 9 );
}


  void
on_macro10_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  Select_Macro( 10 );
}


  void
on_error_dialog_destroy(
	GObject  *object,
	gpointer  user_data)
{
  error_dialog = NULL;
}


  void
on_error_ok_button_clicked(
	GtkButton *button,
	gpointer   user_data)
{
  gtk_widget_destroy( error_dialog );
}


  gboolean
on_error_dialog_delete_event(
	GtkWidget *widget,
	GdkEvent  *event,
	gpointer   user_data)
{
  return( TRUE );
}


  gboolean
on_bad_entry_dialog_delete_event(
	GtkWidget *widget,
	GdkEvent  *event,
	gpointer   user_data)
{
  return( TRUE );
}


  void
on_scope_configure_event(
	GtkWidget    *widget,
	GdkEventConfigure *event,
	gpointer      user_data)
{
 Create_Scope_Pixbuf( event->width, event->height );
}


  void
on_txvol_spinbutton_value_changed(
	GtkSpinButton *spinbutton,
	gpointer       user_data)
{
  char mesg[MESG_SIZE];
  int error, spin_val;

  spin_val = gtk_spin_button_get_value_as_int(spinbutton);

  /* If tcvr is K2 or K3, set power output setting */
  if( (rc_data.tcvr_type == K2) || (rc_data.tcvr_type == K3) )
  {
	rc_data.tx_output = spin_val;
	if( isFlagSet(CAT_SETUP) )
	  Set_Tx_Power( rc_data.tx_output, 0 );
	rc_data.pbk_lev = K3_PBK_LEVEL;
  }
  else rc_data.pbk_lev = spin_val;

  /* Set line output level */
  Set_Playback_Level( rc_data.pbk_lev, mesg, &error );
}


  void
on_rxvol_spinbutton_value_changed(
	GtkSpinButton *spinbutton,
	gpointer       user_data)
{
  char mesg[MESG_SIZE];
  int error;

  rc_data.cap_lev = gtk_spin_button_get_value_as_int(spinbutton);
  if( !Set_Capture_Level(rc_data.cap_lev, mesg, &error) )
  {
	Strlcat( mesg, snd_strerror(error), sizeof(mesg) );
	fprintf( stderr, "%s", mesg );
  }
}


  void
on_afc_checkbutton_toggled(
	GtkToggleButton *togglebutton,
	gpointer         user_data)
{
  if( gtk_toggle_button_get_active(togglebutton) )
	SetFlag( ENABLE_AFC );
  else
  {
	ClearFlag( ENABLE_AFC );
	freq_error = 0.0;
  }
}

